SystemOrganization addCategory: #'Websela-Bootstrap'!
SystemOrganization addCategory: #'Websela-Bootstrap-Tests'!

TestCase subclass: #MCWebServerTest
	instanceVariableNames: 'mcserver'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Websela-Bootstrap-Tests'!

!MCWebServerTest methodsFor: 'as yet unclassified' stamp: 'chc 11/26/2013 14:55'!
setUp
	mcserver := MCWebServer port: 9912.
	mcserver basicStart! !

!MCWebServerTest methodsFor: 'as yet unclassified' stamp: 'chc 11/26/2013 14:42'!
tearDown 
	mcserver := nil! !

!MCWebServerTest methodsFor: 'as yet unclassified' stamp: 'chc 11/26/2013 15:30'!
testGETRoutesToSeaside

	self assert: ((WebClient new httpGet: 'http://localhost:8080') class = WebResponse)! !

!MCWebServerTest methodsFor: 'as yet unclassified' stamp: 'chc 11/26/2013 15:16'!
testPUTLoadsFileAuth! !

!MCWebServerTest methodsFor: 'as yet unclassified' stamp: 'chc 11/26/2013 15:16'!
testPUTLoadsFileNoAuth! !

WAWebServerAdaptor subclass: #MCWebServer
	instanceVariableNames: 'directory isAuthenticated'
	classVariableNames: 'Default'
	poolDictionaries: ''
	category: 'Websela-Bootstrap'!

!MCWebServer commentStamp: 'chc 3/27/2013 14:42' prior: 0!
I am MCWebServer.
I implement an HTTP server specialized for the Monticello protocol.

(MCWebServer reset default)
	directory: (FileDirectory  default / 'package-cache');
	addUser: 'john' withPassword: 'secret';
	listenOn: 8181.

Instance Variables:
	webServer <WebServer>
	directory <FileDirectory>
	isAuthenticated <Boolean>!

!MCWebServer class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/22/2010 11:22'!
default
	Default ifNil: [ Default := self new ].
	^ Default! !

!MCWebServer class methodsFor: 'accessing' stamp: 'chc 5/18/2013 12:06'!
initialize
	"MCWebServer initialize"
	Smalltalk addToStartUpList: self.  ! !

!MCWebServer class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/22/2010 12:08'!
reset
	Default ifNotNil: [ Default destroy. Default := nil ].! !

!MCWebServer class methodsFor: 'accessing' stamp: 'chc 11/26/2013 14:27'!
startUp
	| adap port passwd username |

	Smalltalk extractParameters	
	at: 'PORT' ifPresent: [ :value | value isEmptyOrNil ifFalse: [ port := value asInteger ] ];
	at: 'PASSWD' ifPresent: [ :value | value isEmptyOrNil ifFalse: [ passwd := value ] ];
	at: 'USERNAME' ifPresent: [ :value | value isEmptyOrNil ifFalse: [ username := value  ] ].
				
	adap := MCWebServer port: port.
	adap basicStart.
	adap directory: FileDirectory default.
	adap server passwordAt: username realm: self realm  put: passwd! !

!MCWebServer methodsFor: 'accessing' stamp: 'chc 11/25/2013 15:00'!
addUser: user withPassword: password
	self isAuthenticated: true.
Transcript cr; show: 'MCWebServer: ' , self realm.
	self server
		passwordAt: user 
		realm: self realm
		put: password ! !

!MCWebServer methodsFor: 'private' stamp: 'chc 11/22/2013 15:12'!
basicStart
	"Start the WebServer instance"
	server := WebServer new.
	server addService: '/' action: [ :request | self dispatchWebRequest: request ]
		methods: #(GET PUT).
	server certName: certName.
	server listenOn: self port.! !

!MCWebServer methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/22/2010 11:28'!
destroy
	self webServer destroy ! !

!MCWebServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/22/2010 11:27'!
directory
	^ directory! !

!MCWebServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/22/2010 14:44'!
directory: aFileDirectory
	directory := aFileDirectory! !

!MCWebServer methodsFor: 'private' stamp: 'chc 11/26/2013 15:04'!
dispatchWebRequest: request
	request isGetRequest ifTrue: [
			self process: request
		 ].
	request isPutRequest ifTrue: [
		"self server
			authenticate: request 
			realm: self realm 
			methods: #(basic) 
			do: [ ^ "self handlePutMczEntry: request "]"
		 ]! !

!MCWebServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/28/2010 19:39'!
handleGetMczEntry: request
	| name |
	name := request url allButFirst.
	((self isValidMczName: name) and: [ self directory fileExists: name ]) 
		ifTrue: [
			self directory readOnlyFileNamed: name do: [ :file | | response |
				file binary.
				response := request newResponse protocol: 'HTTP/1.1' code: 200.
				response headerAt: 'Content-Type' put: 'application/octet-stream'.
				request sendResponse: response contentStream: file size: file size.] ]
		ifFalse: [ request sendNotFound ]! !

!MCWebServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/28/2010 19:25'!
handleListMczEntries: request
	request send200Response: self repositoryListing contentType: 'text/html'! !

!MCWebServer methodsFor: 'private' stamp: 'chc 11/26/2013 15:41'!
handlePutMczEntry: request
	| name |
	name := request url allButFirst.
	((self isValidMczName: name) and: [ (self directory fileExists: name) not ]) 
		ifTrue: [
			self directory newFileNamed: name do: [ :file | 
				file binary.
				request streamFrom: request stream to: file size: request contentLength progress: nil.
				request sendCreated.
				MczInstaller installFileNamed: file name ] ]
		ifFalse: [ request sendBadRequest ]! !

!MCWebServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/22/2010 14:18'!
isAuthenticated
	^ isAuthenticated! !

!MCWebServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/22/2010 14:45'!
isAuthenticated: aBoolean
	isAuthenticated := aBoolean! !

!MCWebServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/22/2010 13:51'!
isValidMczName: string
	^ string isString 
		and: [ string isEmpty not
			and: [ (string endsWith: '.mcz')
				and: [ (string includes: $/) not ] ] ]! !

!MCWebServer methodsFor: 'initialize-release' stamp: 'chc 5/19/2013 12:01'!
listenOn: port
	self server listenOn: port! !

!MCWebServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/22/2010 12:53'!
mczEntries
	^ (self directory fileNames select: [ :each | each endsWith: '.mcz' ])
		asSortedCollection! !

!MCWebServer methodsFor: 'accessing' stamp: 'chc 11/26/2013 14:27'!
realm
	^ 'Websela'! !

!MCWebServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/22/2010 12:42'!
repositoryListing
	^ String streamContents: [ :str |
		str nextPutAll: '<html><head><title>'; nextPutAll: self realm; nextPutAll: '</title></head>'.
		str nextPutAll: '<body><h1>'; nextPutAll: self realm; nextPutAll: '</h1>'.
		str nextPutAll: '<ul>'.
		self mczEntries do: [ :each | 
			str nextPutAll: '<li><a href="'; nextPutAll: each; nextPutAll: '">'; nextPutAll: each; nextPutAll: '</a></li>'].
		str nextPutAll: '</ul></body></html>' ]! !

!MCWebServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/22/2010 11:26'!
webServer
	^ webServer! !

!MCWebServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/22/2010 14:45'!
webServer: aWebServer
	webServer := aWebServer! !

!WAWebServerAdaptor methodsFor: '*websela-boostrap' stamp: 'chc 11/27/2013 15:07'!
server
	^server! !

!WAWebServerAdaptor methodsFor: '*websela-boostrap' stamp: 'chc 11/27/2013 15:07'!
server: anObject
	server := anObject! !
MCWebServer initialize!
